/*
ip_to_country copyright Jan Panteltje 2007-always.
 
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License ONLY.
 
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
 
You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/* SET TABS TO 4 TO READ THIS SOURCE */
                                             
                                             


#include <stdlib.h>
#include <stdio.h> 
#include <stdint.h>
#include <string.h>
#include <stddef.h>
#include <getopt.h>
#include <ctype.h>
#include <sys/types.h>
#include <pwd.h>
#include <unistd.h>


#define PROG_VERSION "0.2"
#define READSIZE 256

int debug_flag;


int readline(FILE *file, char *contents)
{
int a, c, i;

if(debug_flag)
	{
//	fprintf(stderr, "readline(): arg file=%p\n", file);
	}

for(i = 0; i < READSIZE - 1; i++)
	{
	while(1)
		{
		c = getc(file);
		a = ferror(file);
		if(a)
			{
			perror("readline():");
			continue;
			}
		break;
		}
	if(feof(file) )
		{
		fclose(file);
		contents[i] = 0;/* EOF marker */
		return(EOF);
		}
	if(c == '\n')
		{
		contents[i] = 0;/* string termination */
		return(1);/* end of line */
		} 
	contents[i] = c;
	}
/*
mmm since we are here, the line must be quite long, possibly something
is wrong.
Since I do not always check for a return 0 in the use uf this function,
just to be safe, gona force a string termination.
This prevents stack overflow, and variables getting overwritten.
*/
contents[i] = 0;/* force string termination */

if(debug_flag)
	{
	fprintf(stderr, "readline(): line to long, returning 0 contents=%s\n", contents);
	}

return 0;
} /* end function readline */


void print_usage()
{
if(debug_flag)
	{
	fprintf(stderr, "print_usage(): arg none\n");
	}

fprintf(stderr,
"\nPanteltje ip_to_country %s\n\
Usage:\nip_to_country: [-d database] [-h] [-i ip] [-v]\n\
-d database    path to database file, default ~/ip-to-country.csv.\n\
-h             help (this help.\n\
-i ipaddress   IP address as dotted quad, like n1.n2.n3.n4\n\
-v             debug, prints variables, functions and arguments.\n\
\n", PROG_VERSION);
} /* end function print_usage */


char *strsave(char *s)
{
char *ptr;

ptr = (char *)malloc(strlen(s) + 1);
if(! ptr) return 0;

strcpy(ptr, s);

return ptr;
} /* end function strsave */


int main(int argc, char **argv)
{
int a, b;
uint32_t ip;
FILE * fptr;
char text[256];
char ab[20];
char country[256];
uint32_t min, max;
int line;
const char delimiters[] = ",\n";
char *token;
int a1, a2, a3, a4;
char *database_filename = 0;
struct passwd *userinfo;
char *home_dir;
char temp[4096];

/* get user info */
userinfo = getpwuid(getuid() );

/* get home directory */ 
home_dir = strsave(userinfo -> pw_dir);
if(! home_dir)
	{
	fprintf(stderr, "xdipo: main(): could allocate space for home_dir, aborting.\n");

	exit(1);
	}

sprintf(temp, "%s/ip-to-country.csv", home_dir);
database_filename = strsave(temp);
if(! database_filename)
	{
	fprintf(stderr, "ip_to_country: could not allocate space for database filename, aborting.\n");

	exit(1);	
	}
	
b = -1;
while(1)
	{
	a = getopt(argc, argv, "d:hi:v");
	if(a == -1) break;

	switch(a)
		{
		case 'd':
			free(database_filename);
			database_filename = strsave(optarg);			
			if(! database_filename)
				{
				fprintf(stderr, "ip_to_country: could not allocate space for database filename %s, aborting.\n", optarg);				

				exit(1);
				}

			break;
		case 'h':
			print_usage();
			exit(1);
			break;
		case 'i':
			b = sscanf(optarg, "%d.%d.%d.%d", &a1, &a2, &a3, &a4);
			break;
		case 'v':
			debug_flag = 1;
			break;
        case -1:
        	break;
		case '?':
			if (isprint(optopt) )
 				{
 				fprintf(stdout, "Unknown option `-%c'.\n", optopt);
 				}
			else
				{
				fprintf(stdout, "Unknown option character `\\x%x'.\n",\
				optopt);
				}
			print_usage();

			exit(1);
			break;			
		default:
			print_usage();

			exit(1);
			break;
		} /* end switch a */
	} /* end while getopt() */

if(! database_filename)
	{
	fprintf(stderr, "ip_to_country: main(): no database_filename specified, aborting.\n");
	
	exit(1);
	}

if(b == -1)
	{
	print_usage();

	fprintf(stderr, "ip_to_country: main(): need IP address as -i xxx.xxx.xxx.xxx , aborting.\n");

	exit(1);
	}

if(b != 4)
	{
	fprintf(stderr, "ip_to_country: main(): incorrect IP address syntax, %d missing fields, use 0-255.0-255.0-255.0-255 aborting.\n", 4 - b);

	exit(1);	
	}

a = 0;
if( (a1 < 0) || (a1 > 255) )
	{	
	a = 1;
	}
if( (a2 < 0) || (a2 > 255) )
	{	
	a = 2;
	}
if( (a3 < 0) || (a3 > 255) )
	{	
	a = 3;
	}
if( (a4 < 0) || (a4 > 255) )
	{	
	a = 4;
	}

if(a)
	{
	fprintf(stderr, "ip_to_country: main(): incorrect IP address syntax in field %d, use 0-255.0-255.0-255.0-255 aborting.\n", a);
	
	exit(1);	
	}

ip = (a1 << 24) | (a2 << 16) | (a3 << 8) | a4;
if(debug_flag)
	{
	fprintf(stderr, "ip_to_country: IP=%ul\n", ip);
	}

fptr = fopen(database_filename, "r");
if(! fptr)
	{
	fprintf(stderr, "ip_to_country: could not open database cvs file %s for read, aborting.\n", database_filename);	

	exit(1);
	}

line = 1;
while(1)
	{
	a = readline(fptr, text);
	if(a == EOF)
		{
		fprintf(stdout, "not found.\n");

		exit(1);
		}

	/*	"1372164096","1372168191","DE","DEU","GERMANY" */

	token = strtok (text, delimiters);
	if(! token)
		{
		fprintf(stderr, "ip_to_country: file format error at line %d in database cvs file %s, aborting.\n", line, database_filename); 
		
		exit(1);
		}
	min = atoll(token + 1);
	if(debug_flag)
		{		
		fprintf(stderr, "min=%ul\n", min);
		}

	token = strtok (NULL, delimiters);
	if(! token)
		{
		fprintf(stderr, "ip_to_country: file format error at line %d in database cvs file %s, aborting.\n", line, database_filename); 
		
		exit(1);
		}
	max = atoll(token + 1);
	if(debug_flag)
		{
		fprintf(stderr, "max=%ul\n", max);
		}
		
	token = strtok (NULL, delimiters);
	if(! token)
		{
		fprintf(stderr, "ip_to_country: file format error at line %d in database cvs file %s, aborting.\n", line, database_filename); 
		
		exit(1);
		}
	strcpy(ab, token);	

	token = strtok (NULL, delimiters);
	if(! token)
		{
		fprintf(stderr, "ip_to_country: file format error at line %d in database cvs file %s, aborting.\n", line, database_filename); 
		
		exit(1);
		}
	// not used 

	token = strtok (NULL, delimiters);
	if(! token)
		{
		fprintf(stderr, "ip_to_country: file format error at line %d in database cvs file %s, aborting.\n", line, database_filename); 
		
		exit(1);
		}
	strcpy(country, token);

	if( (ip >= min) && (ip <= max) )
		{
		fprintf(stdout, "ip=%d.%d.%d.%d (%u) %s %s\n", a1, a2, a3, a4, ip, ab, country);

		/* found */		
		exit(0);
		}
	
	line++;
	} /* end while read database */

/* never here */
exit(1);
} /* end function main */


  